/*
m_risset_filter - multi-bandpass filter with its frequency and amplitude
	modulated by a Risset glissando

DESCRIPTION
This is a multi-bandpass filter, where the centre frequencies and amplitudes
recreate a Risset-Shepard glissando.

SYNTAX
aout m_risset_filter ain, klevel, krate, ioctave, kcentre, kq, kdirection, \
	iwindow, irise

INITIALIZATION
ioctave - The octave spread from lowest to highest frequency (3-10)
iwindow - Windown function for the amplitude change, best use GEN20 with type
	6, max 1 and option 1 (higher values may cause peaks when direction is
	negative!)
irise - Function modulating the frequency, best take GEN5 with somet;hing like:
	f1 0 16384 -5 1 16384 10

PERFORMANCE
ain - Audio input signal to be filtered
klevel - Output level of the filter (recommendation: >0, decrease with
	increasing number of octaves and increasing q values)
krate - Modulation rate or tempo of the risset glissando in Hertz
kcentre - Centre frequency of the Risset Glissando in Hertz
kq - bandwidth or Q of the bandpass filters in Hertz
kdirection - Direction (up or down) of the Risset glissando
	<0: down
	>= 0: up

CREDITS
Author: Jeanette C., thanks to the Csound community for their constant support,
in particular Oeyvind Brandstegg for solving a few issues.
*/

opcode m_risset_filter, a, akkikkkii
	ain, klevel, krate, ioctave, kcentre, kq, kdirection, iwindow, irise xin
	
	; Limit the octave between 3 and 10 and make sure we only get int numbers
	ioctave limit ceil(ioctave), 3, 10

	; Calculate correct rate
	krate = krate / ioctave

	; Set the start frequency
	kstart_freq = kcentre * semitone(-ioctave/2 * 12)

	; set up direction factor
	if kdirection < 0 then
		kdirfac = -1
	else
		kdirfac = 1
	endif

	; Set up the phasor and phase shift increments for the filters
	kphs phasor krate
	iphs_shift init 1/ioctave
	afilt m_risset_int ain, kstart_freq, kq, ioctave, kdirfac, kphs, \
		iphs_shift, 0, iwindow, irise

	; Scale the output

	xout afilt * klevel
endop
